@@ -7,17 +7,17 @@
  *  codec.
  *
  *  libdv is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your
+ *  under the terms of the GNU Lesser Public License as published by
+ *  the Free Software Foundation; either version 2.1, or (at your
  *  option) any later version.
  *   
  *  libdv is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  General Public License for more details.
+ *  Lesser Public License for more details.
  *   
- *  You should have received a copy of the GNU General Public License
- *  along with GNU Make; see the file COPYING.  If not, write to
+ *  You should have received a copy of the GNU Lesser Public License
+ *  along with libdv; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
  *  The libdv homepage is http://libdv.sourceforge.net/.  
@@ -38,7 +38,6 @@
 #include <sched.h>
 #include <sys/resource.h>
 #include <string.h>
-
 #include <pthread.h>
 #include <signal.h>
 
@@ -46,14 +45,13 @@
 #include <popt.h>
 #endif
 
-#include <malloc.h>
+#include <stdlib.h>
 
 static long cip_n_ntsc = 2436;
 static long cip_d_ntsc = 38400;
 static long cip_n_pal = 1;
 static long cip_d_pal = 16;
 static long syt_offset = 11000;
-static int video1394_version = 2;
 
 #define TARGETBUFSIZE   320
 #define MAX_PACKET_SIZE 512
@@ -67,36 +65,13 @@ enum {
 	VIDEO1394_BUFFER_READY
 };
 
-enum {
-	VIDEO1394_LISTEN_CHANNEL = 0,
-	VIDEO1394_UNLISTEN_CHANNEL,
-	VIDEO1394_LISTEN_QUEUE_BUFFER,
-	VIDEO1394_LISTEN_WAIT_BUFFER,
-	VIDEO1394_TALK_CHANNEL,
-	VIDEO1394_UNTALK_CHANNEL,
-	VIDEO1394_TALK_QUEUE_BUFFER,
-	VIDEO1394_TALK_WAIT_BUFFER
-};
-
 #define VIDEO1394_SYNC_FRAMES          0x00000001
 #define VIDEO1394_INCLUDE_ISO_HEADERS  0x00000002
 #define VIDEO1394_VARIABLE_PACKET_SIZE 0x00000004
 
 struct video1394_mmap
 {
-	unsigned int channel;
-	unsigned int sync_tag;
-	unsigned int nb_buffers;
-	unsigned int buf_size;
-        /* For VARIABLE_PACKET_SIZE: Maximum packet size */
-	unsigned int packet_size; 
-	unsigned int fps;
-	unsigned int flags;
-};
-
-struct video1394_mmap_v2
-{
-	unsigned int channel;
+	int channel;
 	unsigned int sync_tag;
 	unsigned int nb_buffers;
 	unsigned int buf_size;
@@ -120,9 +95,28 @@ struct video1394_wait
 {
 	unsigned int channel;
 	unsigned int buffer;
-        struct timeval filltime;        /* time of buffer full */
+	struct timeval filltime;        /* time of buffer full */
 };
 
+#define VIDEO1394_LISTEN_CHANNEL		\
+	_IOWR('#', 0x10, struct video1394_mmap)
+#define VIDEO1394_UNLISTEN_CHANNEL		\
+	_IOW ('#', 0x11, int)
+#define VIDEO1394_LISTEN_QUEUE_BUFFER	\
+	_IOW ('#', 0x12, struct video1394_wait)
+#define VIDEO1394_LISTEN_WAIT_BUFFER	\
+	_IOWR('#', 0x13, struct video1394_wait)
+#define VIDEO1394_TALK_CHANNEL		\
+	_IOWR('#', 0x14, struct video1394_mmap)
+#define VIDEO1394_UNTALK_CHANNEL		\
+	_IOW ('#', 0x15, int)
+#define VIDEO1394_TALK_QUEUE_BUFFER 	\
+	_IOW ('#', 0x16, size_t)
+#define VIDEO1394_TALK_WAIT_BUFFER		\
+	_IOW ('#', 0x17, struct video1394_wait)
+#define VIDEO1394_LISTEN_POLL_BUFFER	\
+	_IOWR('#', 0x18, struct video1394_wait)
+
 static int cap_start_frame = 0;
 static int cap_num_frames = 0xfffffff;
 static int cap_verbose_mode;
@@ -339,14 +333,13 @@ int capture_raw(const char* filename, int channel, int nbuffers,
 		int start_frame, int end_frame, int verbose_mode,
 		char *device)
 {
-        int viddev;
-        unsigned char *recv_buf;
-        struct video1394_mmap v;
-        struct video1394_mmap_v2 v2;
-        struct video1394_wait w;
+	int viddev;
+	unsigned char *recv_buf;
+	struct video1394_mmap v;
+	struct video1394_wait w;
 	int unused_buffers;
-        unsigned char outbuf[2*65536];
-        int outbuf_used = 0;
+	unsigned char outbuf[2*65536];
+	int outbuf_used = 0;
 
 	cap_start_frame = start_frame;
 	cap_num_frames = end_frame - start_frame;
@@ -355,57 +348,38 @@ int capture_raw(const char* filename, int channel, int nbuffers,
 	if (!filename || strcmp(filename, "-") == 0) {
 		dst_fp = stdout;
 	} else {
-		dst_fp = fopen(filename, "rb");
+		dst_fp = fopen(filename, "wb");
 		if (!dst_fp) {
 			perror("fopen input file");
 			return(-1);
 		}
 	}
 
-        if ((viddev = open(device, O_RDWR)) < 0) {
+	if ((viddev = open(device, O_RDWR)) < 0) {
 		perror("open video1394 device");
-                return -1;
-        }
+		return -1;
+	}
 
 	v.channel = channel;
 	v.sync_tag = 0;
 	v.nb_buffers = nbuffers;
 	v.buf_size = VBUF_SIZE; 
 	v.packet_size = MAX_PACKET_SIZE;
+	v.syt_offset = syt_offset;
 	v.flags = VIDEO1394_INCLUDE_ISO_HEADERS;
 	w.channel = v.channel;
                       
-	switch (video1394_version) {
-	case 1:
-		if (ioctl(viddev, VIDEO1394_LISTEN_CHANNEL, &v) < 0) {
-			perror("VIDEO1394_LISTEN_CHANNEL");
-			return -1;
-		}
+	if (ioctl(viddev, VIDEO1394_LISTEN_CHANNEL, &v) < 0) {
+		perror("VIDEO1394_LISTEN_CHANNEL");
+		return -1;
+	}
 
-		break;
-	case 2:
-		v2.channel = v.channel;
-		v2.sync_tag = v.sync_tag;
-		v2.nb_buffers = v.nb_buffers;
-		v2.buf_size = v.buf_size;
-		v2.packet_size = v.packet_size;
-		v2.syt_offset = syt_offset;
-		v2.flags = v.flags;
-
-		if (ioctl(viddev, VIDEO1394_LISTEN_CHANNEL, &v2) < 0) {
-			perror("VIDEO1394_LISTEN_CHANNEL");
-			return -1;
-		}
-
-		break;
-	};
-
-        if ((recv_buf = (unsigned char *) mmap(
-                0, v.nb_buffers*v.buf_size, PROT_READ|PROT_WRITE,
-                MAP_SHARED, viddev, 0)) == (unsigned char *)-1) {
-                perror("mmap videobuffer");
-                return -1;
-        }
+	if ((recv_buf = (unsigned char *) mmap(
+			0, v.nb_buffers*v.buf_size, PROT_READ|PROT_WRITE,
+			MAP_SHARED, viddev, 0)) == (unsigned char *)-1) {
+		perror("mmap videobuffer");
+		return -1;
+	}
 
 	pthread_mutex_init(&queue_mutex, NULL);
 	pthread_mutex_init(&wakeup_mutex, NULL);
@@ -419,68 +393,65 @@ int capture_raw(const char* filename, int channel, int nbuffers,
         unused_buffers = v.nb_buffers;
         w.buffer = 0;
        
-        while (cap_num_frames != 0) {
-		struct video1394_wait wcopy;
-		unsigned char * curr;
-                int ofs;
-
-                while (unused_buffers--) {
-                        unsigned char * curr = recv_buf+ v.buf_size * w.buffer;
-
-                        memset(curr, 0, v.buf_size);
-                        
-			wcopy = w;
-
-                        if (ioctl(viddev,VIDEO1394_LISTEN_QUEUE_BUFFER,
-				  &wcopy) < 0) {
-                                perror("VIDEO1394_LISTEN_QUEUE_BUFFER");
-                        }
-                        w.buffer++;
-                        w.buffer %= v.nb_buffers;
-                }
-                wcopy = w;
-                if (ioctl(viddev, VIDEO1394_LISTEN_WAIT_BUFFER, &wcopy) < 0) {
-                        perror("VIDEO1394_LISTEN_WAIT_BUFFER");
-                }
-                curr = recv_buf + v.buf_size * w.buffer;
-                ofs = 0;
-
-                while (ofs < VBUF_SIZE) {
-                        while (outbuf_used < 4 && ofs < VBUF_SIZE) {
-                                outbuf[outbuf_used++] = curr[ofs++];
-                        }
-                        if (ofs != VBUF_SIZE) {
-                                int len = outbuf[2] + (outbuf[3] << 8) + 8;
-                                if (ofs + len - outbuf_used > VBUF_SIZE) {
-                                        memcpy(outbuf + outbuf_used, curr+ofs, 
-                                               VBUF_SIZE - ofs);
-                                        outbuf_used += VBUF_SIZE - ofs;
-                                        ofs = VBUF_SIZE;
-                                } else {
-                                        memcpy(outbuf + outbuf_used,
-                                               curr + ofs, len - outbuf_used);
-                                        ofs += len - outbuf_used;
-                                        outbuf_used = 0;
-                                        handle_packet(outbuf, len - 8);
-                                }
-                        }
-                }
-
-                unused_buffers = 1;
-        }
-
-	if (broken_frames) {
-		fprintf(stderr, "\nCaptured %d broken frames!\n",
-			broken_frames);
+	while (cap_num_frames != 0) {
+	struct video1394_wait wcopy;
+	unsigned char * curr;
+	int ofs;
+	
+	while (unused_buffers--) {
+		unsigned char * curr = recv_buf+ v.buf_size * w.buffer;
+		
+		memset(curr, 0, v.buf_size);
+		
+		wcopy = w;
+		
+		if (ioctl(viddev,VIDEO1394_LISTEN_QUEUE_BUFFER, &wcopy) < 0) {
+			perror("VIDEO1394_LISTEN_QUEUE_BUFFER");
+		}
+		w.buffer++;
+		w.buffer %= v.nb_buffers;
+	}
+	wcopy = w;
+	if (ioctl(viddev, VIDEO1394_LISTEN_WAIT_BUFFER, &wcopy) < 0) {
+		perror("VIDEO1394_LISTEN_WAIT_BUFFER");
+	}
+	curr = recv_buf + v.buf_size * w.buffer;
+	ofs = 0;
+	
+	while (ofs < VBUF_SIZE) {
+		while (outbuf_used < 4 && ofs < VBUF_SIZE) {
+			outbuf[outbuf_used++] = curr[ofs++];
+		}
+		if (ofs != VBUF_SIZE) {
+			int len = outbuf[2] + (outbuf[3] << 8) + 8;
+			if (ofs + len - outbuf_used > VBUF_SIZE) {
+				memcpy(outbuf + outbuf_used, curr+ofs, 
+				VBUF_SIZE - ofs);
+				outbuf_used += VBUF_SIZE - ofs;
+				ofs = VBUF_SIZE;
+			} else {
+				memcpy(outbuf + outbuf_used,
+				curr + ofs, len - outbuf_used);
+				ofs += len - outbuf_used;
+				outbuf_used = 0;
+				handle_packet(outbuf, len - 8);
+			}
+		}
+	}
+	unused_buffers = 1;
 	}
 
-        munmap(recv_buf, v.nb_buffers * v.buf_size);
-        
-        if (ioctl(viddev, VIDEO1394_UNLISTEN_CHANNEL, &v.channel)<0) {
-                perror("VIDEO1394_UNLISTEN_CHANNEL");
-        }
+	if (broken_frames) {
+		fprintf(stderr, "\nCaptured %d broken frames!\n", broken_frames);
+	}
 
-        close(viddev);
+	munmap(recv_buf, v.nb_buffers * v.buf_size);
+	
+	if (ioctl(viddev, VIDEO1394_UNLISTEN_CHANNEL, &v.channel)<0) {
+		perror("VIDEO1394_UNLISTEN_CHANNEL");
+	}
+	
+	close(viddev);
 
 	pthread_join(file_io_thread, NULL);
 
@@ -488,13 +459,14 @@ int capture_raw(const char* filename, int channel, int nbuffers,
 		fclose(dst_fp);
 	}
 
-        return 0;
+	return 0;
 }
 
 /* ------------------------------------------------------------------------
    - sender
    ------------------------------------------------------------------------ */
 
+static const char*const* src_filenames;
 static FILE* src_fp;
 static int is_eof = 0;
 static unsigned char * underrun_data_frame = NULL;
@@ -531,6 +503,27 @@ void sig_int_send_handler(int signum)
 	is_eof = 1;
 }
 
+static int
+open_next_input(void)
+{
+	if (src_fp && src_fp != stdin) {
+		fclose (src_fp);
+		src_fp = NULL;
+	}
+	if (!src_filenames || !*src_filenames)
+		return 1;
+	if (!strcmp (*src_filenames++, "-"))
+		src_fp = stdin;
+	else {
+		src_fp = fopen(src_filenames[-1], "rb");
+		if (!src_fp) {
+			perror("fopen input file");
+			return 1;
+		}
+	}
+	return 0;
+}
+
 
 void fill_buf_queue(int fire)
 {
@@ -538,8 +531,11 @@ void fill_buf_queue(int fire)
 
 	while ((f = get_free_block()) != NULL) {
 		int isPAL;
+again:
 		if (read_frame(src_fp, f->data, &isPAL) < 0) {
-			is_eof = 1;
+			is_eof = open_next_input ();
+			if (!is_eof)
+				goto again;
 			push_back(&free_list, f);
 			return;
 		}
@@ -636,39 +632,39 @@ static int fill_buffer(unsigned char* targetbuf, unsigned int * packet_sizes)
 		cip_counter = cip_n;
 	}
 
-        for (i = 0; i < TARGETBUFSIZE && vdata < frame_size; i++) {
-                unsigned char* p = targetbuf;
-                int want_sync = 0;
+	for (i = 0; i < TARGETBUFSIZE && vdata < frame_size; i++) {
+		unsigned char* p = targetbuf;
+		int want_sync = 0;
 		cip_counter += cip_n;
-
+		
 		if (cip_counter > cip_d) {
 			want_sync = 1;
 			cip_counter -= cip_d;
 		}
-
-                *p++ = 0x01; /* Source node ID ! */
-                *p++ = 0x78; /* Packet size in quadlets (480 / 4) */
-                *p++ = 0x00;
-                *p++ = continuity_counter;
-                
-                *p++ = 0x80; /* const */
+		
+		*p++ = 0x01; /* Source node ID ! */
+		*p++ = 0x78; /* Packet size in quadlets (480 / 4) */
+		*p++ = 0x00;
+		*p++ = continuity_counter;
+		
+		*p++ = 0x80; /* const */
 		*p++ = isPAL ? 0x80 : 0x00;
-                *p++ = 0xff; /* timestamp */
-                *p++ = 0xff; /* timestamp */
-                
-                /* Timestamping is now done in the kernel driver! */
-                if (!want_sync) { /* video data */
-                        continuity_counter++;
-
-                        memcpy(p, frame + vdata, 480);
-                        p += 480;
-                        vdata += 480;
-                }
-
-                *packet_sizes++ = p - targetbuf;
-                targetbuf += MAX_PACKET_SIZE;
-        }
-        *packet_sizes++ = 0;
+		*p++ = 0xff; /* timestamp */
+		*p++ = 0xff; /* timestamp */
+		
+		/* Timestamping is now done in the kernel driver! */
+		if (!want_sync) { /* video data */
+			continuity_counter++;
+			
+			memcpy(p, frame + vdata, 480);
+			p += 480;
+			vdata += 480;
+		}
+		
+		*packet_sizes++ = p - targetbuf;
+		targetbuf += MAX_PACKET_SIZE;
+	}
+	*packet_sizes++ = 0;
 
 	push_back(&free_list, f_node);
 
@@ -679,34 +675,33 @@ static int fill_buffer(unsigned char* targetbuf, unsigned int * packet_sizes)
         return 0;
 }
 
-int send_raw(const char* filename, int channel, int nbuffers, int start_frame,
-	     int end_frame, int verbose_mode, 
-	     const char * underrun_data_filename, char *device)
+int send_raw(const char*const* filenames, int channel, int nbuffers,
+		int start_frame, int end_frame, int verbose_mode, 
+		const char * underrun_data_filename, char *device)
 {
-        int viddev;
-        unsigned char *send_buf;
-        struct video1394_mmap v;
-        struct video1394_mmap_v2 v2;
-        struct video1394_queue_variable w;
+	int viddev;
+	unsigned char *send_buf;
+	struct video1394_mmap v;
+	struct video1394_queue_variable w;
 	int unused_buffers;
 	int got_frame;
-        unsigned int packet_sizes[321];
+	unsigned int packet_sizes[321];
 
-	if (!filename || strcmp(filename, "-") == 0) {
+	if ( filenames == NULL )
+		return -1;
+
+	src_filenames = filenames;
+	if (!*src_filenames) {
+		src_filenames = 0;
 		src_fp = stdin;
-	} else {
-		src_fp = fopen(filename, "rb");
-		if (!src_fp) {
-			perror("fopen input file");
-			return -1;
-		}
 	}
-
+	else if (open_next_input ())
+		return -1;
+  
 	if (underrun_data_filename) {
 		FILE * fp = fopen(underrun_data_filename, "rb");
 		if (!fp) {
 			perror("fopen underrun data file");
-			fclose(src_fp);
 			return -1;
 		}
 		underrun_data_frame = (unsigned char*) malloc(144000);
@@ -714,14 +709,13 @@ int send_raw(const char* filename, int channel, int nbuffers, int start_frame,
 			       &underrun_frame_ispal) < 0) {
 			fprintf(stderr, "Short read on reading underrun data "
 				"frame...\n");
-			fclose(src_fp);
 			fclose(fp);
 			return -1;
 		}
 		fclose(fp);
 	}
 
-	if (start_frame > 0) {
+	while (start_frame > 0) {
 		int isPAL,i;
 		unsigned char frame[144000]; /* PAL is large enough... */
 
@@ -731,56 +725,49 @@ int send_raw(const char* filename, int channel, int nbuffers, int start_frame,
 					start_frame - i);
 			}
 			if (read_frame(src_fp, frame, &isPAL) < 0) {
-				return -1;
+				break;
 			}
 		}
+		start_frame -= i;
+		if (!start_frame)
+			break;
+		if (open_next_input ())
+			return -1;
 	}
 
-        if ((viddev = open(device,O_RDWR)) < 0) {
+	if ((viddev = open(device,O_RDWR)) < 0) {
 		perror("open video1394 device");
-                return -1;
-        }
+		if (src_fp && src_fp != stdin)
+			fclose (src_fp);
+		return -1;
+  	}
                       
-        v.channel = channel;
-        v.sync_tag = 0;
-        v.nb_buffers = nbuffers;
-        v.buf_size = TARGETBUFSIZE * MAX_PACKET_SIZE; 
-        v.packet_size = MAX_PACKET_SIZE;
-        v.flags = VIDEO1394_VARIABLE_PACKET_SIZE;
-        w.channel = v.channel;
+	v.channel = channel;
+	v.sync_tag = 0;
+	v.nb_buffers = nbuffers;
+	v.buf_size = TARGETBUFSIZE * MAX_PACKET_SIZE; 
+	v.packet_size = MAX_PACKET_SIZE;
+	v.flags = VIDEO1394_VARIABLE_PACKET_SIZE;
+	v.syt_offset = syt_offset;
+	w.channel = v.channel;
 
-	switch (video1394_version) {
-	case 1:
-		if (ioctl(viddev, VIDEO1394_TALK_CHANNEL, &v) < 0) {
-			perror("VIDEO1394_TALK_CHANNEL");
-			return -1;
-		}
+	if (ioctl(viddev, VIDEO1394_TALK_CHANNEL, &v) < 0) {
+		perror("VIDEO1394_TALK_CHANNEL");
+		close (viddev);
+		if (src_fp && src_fp != stdin)
+			fclose (src_fp);
+		return -1;
+	}
 
-		break;
-	case 2:
-		v2.channel = v.channel;
-		v2.sync_tag = v.sync_tag;
-		v2.nb_buffers = v.nb_buffers;
-		v2.buf_size = v.buf_size;
-		v2.packet_size = v.packet_size;
-		v2.syt_offset = syt_offset;
-		v2.flags = v.flags;
-
-		if (ioctl(viddev, VIDEO1394_TALK_CHANNEL, &v2) < 0) {
-			perror("VIDEO1394_TALK_CHANNEL");
-			return -1;
-		}
-
-		break;
-	};
-
-
-        if ((send_buf = (unsigned char *) mmap(
-                0, v.nb_buffers * v.buf_size, PROT_READ|PROT_WRITE,
-                MAP_SHARED, viddev, 0)) == (unsigned char *)-1) {
-                perror("mmap videobuffer");
-                return -1;
-        }
+	if ((send_buf = (unsigned char *) mmap(
+			0, v.nb_buffers * v.buf_size, PROT_READ|PROT_WRITE,
+			MAP_SHARED, viddev, 0)) == (unsigned char *)-1) {
+		perror("mmap videobuffer");
+		close (viddev);
+		if (src_fp && src_fp != stdin)
+		fclose (src_fp);
+		return -1;
+	}
 
 	if (verbose_mode) {
 		fprintf(stderr, "Filling buffers...\r");
@@ -805,66 +792,63 @@ int send_raw(const char* filename, int channel, int nbuffers, int start_frame,
 		fprintf(stderr, "Transmitting...\r");
 	}
 
-        unused_buffers = v.nb_buffers;
-        w.buffer = 0;
-        got_frame = 1;
-        w.packet_sizes = packet_sizes;
+	unused_buffers = v.nb_buffers;
+	w.buffer = 0;
+	got_frame = 1;
+	w.packet_sizes = packet_sizes;
 	memset(packet_sizes, 0, sizeof(packet_sizes));
 
-        for (;start_frame < end_frame;) {
-                while (unused_buffers--) {
-                        got_frame = (fill_buffer(
-                                send_buf + w.buffer * v.buf_size,
-                                packet_sizes) < 0) ? 0 : 1;
-
-                        if (!got_frame) {
-                                break;
-                        }
-                        if (ioctl(viddev, VIDEO1394_TALK_QUEUE_BUFFER, &w)<0) {
-                                perror("VIDEO1394_TALK_QUEUE_BUFFER");
-                        }
+	for (;start_frame < end_frame;) {
+		while (unused_buffers--) {
+			got_frame = (fill_buffer(
+			send_buf + w.buffer * v.buf_size,
+			packet_sizes) < 0) ? 0 : 1;
+			
+			if (!got_frame) {
+				break;
+			}
+			if (ioctl(viddev, VIDEO1394_TALK_QUEUE_BUFFER, &w)<0) {
+				perror("VIDEO1394_TALK_QUEUE_BUFFER");
+			}
 			if (verbose_mode) {
 				fprintf(stderr, "Sent frame %8d\r",
-					start_frame);
+				start_frame);
 			}
-                        w.buffer ++;
-                        w.buffer %= v.nb_buffers;
+			w.buffer ++;
+			w.buffer %= v.nb_buffers;
 			start_frame++;
-                }
-                if (!got_frame) {
-                        break;
-                }
-                if (ioctl(viddev, VIDEO1394_TALK_WAIT_BUFFER, &w) < 0) {
-                        perror("VIDEO1394_TALK_WAIT_BUFFER");
-                }
-                unused_buffers = 1;
-        }
-
-        w.buffer = (v.nb_buffers + w.buffer - 1) % v.nb_buffers;
-
-        if (ioctl(viddev, VIDEO1394_TALK_WAIT_BUFFER, &w) < 0) {
-                perror("VIDEO1394_TALK_WAIT_BUFFER");
-        }
-
-        munmap(send_buf, v.nb_buffers * v.buf_size);
-        
-        if (ioctl(viddev, VIDEO1394_UNTALK_CHANNEL, &v.channel)<0) {
-                perror("VIDEO1394_UNTALK_CHANNEL");
-        }
-
-        close(viddev);
+		}
+		if (!got_frame) {
+			break;
+		}
+		if (ioctl(viddev, VIDEO1394_TALK_WAIT_BUFFER, &w) < 0) {
+			perror("VIDEO1394_TALK_WAIT_BUFFER");
+		}
+		unused_buffers = 1;
+	}
+  
+	w.buffer = (v.nb_buffers + w.buffer - 1) % v.nb_buffers;
+	
+	if (ioctl(viddev, VIDEO1394_TALK_WAIT_BUFFER, &w) < 0) {
+		perror("VIDEO1394_TALK_WAIT_BUFFER");
+	}
+	
+	munmap(send_buf, v.nb_buffers * v.buf_size);
+	
+	if (ioctl(viddev, VIDEO1394_UNTALK_CHANNEL, &v.channel)<0) {
+		perror("VIDEO1394_UNTALK_CHANNEL");
+	}
+	
+	close(viddev);
 
 	pthread_join(file_io_thread, NULL);
 
-	if (src_fp != stdin) {
-		fclose(src_fp);
-	}
-
-        return 0;
+  return 0;
 }
 
 int rt_raisepri (int pri)
 {
+#ifdef _SC_PRIORITY_SCHEDULING
 	struct sched_param scp;
 
 	/*
@@ -882,6 +866,7 @@ int rt_raisepri (int pri)
 			return (-1);
 		}
 	}
+#endif
 	return (0);
 }
 
@@ -899,18 +884,16 @@ int rt_raisepri (int pri)
 #define DV_CONNECT_OPT_CIP_N_NTSC       7
 #define DV_CONNECT_OPT_CIP_D_NTSC       8
 #define DV_CONNECT_OPT_SYT_OFFSET       9
-#define DV_CONNECT_OPT_VIDEO1394_VERSION 10
-#define DV_CONNECT_OPT_MAX_BUFFERS      11
-#define DV_CONNECT_OPT_UNDERRUN_DATA    12
-#define DV_CONNECT_OPT_AUTOHELP         13
-#define DV_CONNECT_OPT_DEVICE           14
-#define DV_CONNECT_NUM_OPTS             15
+#define DV_CONNECT_OPT_MAX_BUFFERS      10
+#define DV_CONNECT_OPT_UNDERRUN_DATA    11
+#define DV_CONNECT_OPT_AUTOHELP         12
+#define DV_CONNECT_OPT_DEVICE           13
+#define DV_CONNECT_NUM_OPTS             14
 
 
 int main(int argc, const char** argv)
 {
 #if HAVE_LIBPOPT
-	const char* filename;
 	int verbose_mode = 0;
 	int send_mode = 0;
 	int channel = 63;
@@ -1001,14 +984,6 @@ int main(int argc, const char** argv)
                 descrip:    "syt offset (default: 10000 range: 10000-26000)"
         }; /* syt offset */
 
-        option_table[DV_CONNECT_OPT_VIDEO1394_VERSION] = (struct poptOption) {
-                longName:   "video-1394-version", 
-                argInfo:    POPT_ARG_INT, 
-                arg:        &video1394_version,
-                descrip:    "video 1394 version to use 1 "
-		"(older kernels or CVS versions), 2 (current kernels >2.4.9?)"
-        }; /* video 1394 version */
-
         option_table[DV_CONNECT_OPT_MAX_BUFFERS] = (struct poptOption) {
                 longName:   "buffers", 
                 shortName:  'b', 
@@ -1039,7 +1014,7 @@ int main(int argc, const char** argv)
                 shortName:  'd', 
                 argInfo: POPT_ARG_STRING,
                 arg:     &device,
-                descrip: "Specify the video1394 device (default /dev/video1394)",
+                descrip: "Specify the video1394 device (default /dev/video1394/0)",
         }; /* device */
 
         option_table[DV_CONNECT_NUM_OPTS] = (struct poptOption) { 
@@ -1047,7 +1022,7 @@ int main(int argc, const char** argv)
 
         optCon = poptGetContext(NULL, argc, 
                                 (const char **)argv, option_table, 0);
-        poptSetOtherOptionHelp(optCon, "<raw dv file or - for stdin/stdout>");
+        poptSetOtherOptionHelp(optCon, "<raw dv files or '-- -' for stdin/stdout>");
 
         while ((rc = poptGetNextOpt(optCon)) > 0) {
                 switch (rc) {
@@ -1077,24 +1052,22 @@ int main(int argc, const char** argv)
 	}
 	if (buffers < 0) {
 		fprintf(stderr, "Number of buffers should be > 0!\n");
-	        return -1;
+		return -1;
 	}
 
-        filename = poptGetArg(optCon);
-
 	setpriority (PRIO_PROCESS, 0, -20);
 	if (rt_raisepri (1) != 0) {
 		setpriority (PRIO_PROCESS, 0, -20);
 	}
 
 	if ( device == NULL )
-                device = "/dev/video1394";
+		device = "/dev/video1394/0";
 
 	if (send_mode) {
-		send_raw(filename, channel, buffers, start, end, verbose_mode,
-			 underrun_data, device);
+		send_raw(poptGetArgs(optCon), channel, buffers, start, end,
+			verbose_mode, underrun_data, device);
 	} else {
-		capture_raw(filename, channel, buffers, start, end, 
+		capture_raw(poptGetArg(optCon), channel, buffers, start, end, 
 			    verbose_mode, device);
 	}
 	return 0;
@@ -1104,16 +1077,3 @@ int main(int argc, const char** argv)
 #endif
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
