@@ -1,4 +1,4 @@
-/*
+ /*
  *  encode.c
  *
  *     Copyright (C) James Bowman  - July 2000
@@ -8,17 +8,17 @@
  *  codec.
  *
  *  libdv is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your
+ *  under the terms of the GNU Lesser Public License as published by
+ *  the Free Software Foundation; either version 2.1, or (at your
  *  option) any later version.
  *   
  *  libdv is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  General Public License for more details.
+ *  Lesser Public License for more details.
  *   
- *  You should have received a copy of the GNU General Public License
- *  along with GNU Make; see the file COPYING.  If not, write to
+ *  You should have received a copy of the GNU Lesser Public License
+ *  along with libdv; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
  *  The libdv homepage is http://libdv.sourceforge.net/.  
@@ -411,15 +411,18 @@ static inline dv_vlc_entry_t * vlc_encode_orig(int run, int amp, int sign,
 	return ++o;
 }
 
-dv_vlc_entry_t * vlc_encode_lookup;
-unsigned char  * vlc_num_bits_lookup;
+dv_vlc_entry_t * vlc_encode_lookup = NULL;
+unsigned char  * vlc_num_bits_lookup = NULL;
 
 void _dv_init_vlc_encode_lookup(void)
 {
 	int run,amp;
-	vlc_encode_lookup = (dv_vlc_entry_t *) malloc(
-		32768 * 2 * sizeof(dv_vlc_entry_t));
-	vlc_num_bits_lookup = (unsigned char*) malloc(32768);
+	
+	if (vlc_encode_lookup == NULL)
+		vlc_encode_lookup = (dv_vlc_entry_t *) malloc(
+			32768 * 2 * sizeof(dv_vlc_entry_t));
+	if (vlc_num_bits_lookup == NULL)
+		vlc_num_bits_lookup = (unsigned char*) malloc(32768);
 		
 	for (run = 0; run <= 63; run++) {
 		for (amp = 0; amp <= 255; amp++) {
@@ -469,12 +472,13 @@ static unsigned short reorder_248[64] = {
 	14,28,30,44,46,58,60,64
 };
 
+
 void _dv_prepare_reorder_tables(void)
 {
 	int i;
 	for (i = 0; i < 64; i++) {
- 		reorder_88[i]--;
- 		reorder_88[i] *= 2;
+		reorder_88[i]--;
+		reorder_88[i] *= 2;
 		reorder_248[i]--;
 		reorder_248[i] *= 2;
 	}
@@ -1401,7 +1405,7 @@ int dv_encoder_loop(dv_enc_input_filter_t * input,
 			skip_frame_count -= 65536;
 			skipped = 1;
 		}
-		if (output->store(target, audio_info, FALSE, 
+		if (output->store(target, audio_info, FALSE,
 				  isPAL, is16x9,now) < 0) {
 			return -1;
 		}
@@ -1456,16 +1460,7 @@ void dv_show_statistics()
  *  This function allocates memory for you that must be set free using
  *  dv_encoder_free()
  * 
- * @param rem_ntsc_setup A boolean that indicates if 16 should be
- *          subtracted from the luma channel to account for the NTSC
- *          luma setup/platform of 7.5 IRE. The North American NTSC 
- *          standard calls for a 7.5 IRE setup, but DV is based on the
- *          Japanese NTSC standard that uses 0 IRE setup. Therefore,
- *          North American DV equipments adds the 7.5 IRE setup to its
- *          analog output. Without setting this true, computer generated
- *          graphics may appear too bright.
- *          Typically, this should be set false for PAL and Japanese NTSC
- *          users.
+ * @param ignored A retired option.
  * @param clamp_luma A boolean to indicate that luma values should be 
  *          trimmed to their ITU-R 601 legal limits. When this is false, it
  *          preserves superblack and superwhite.
@@ -1475,12 +1470,14 @@ void dv_show_statistics()
  *          Typically this should be set true.
  */
 dv_encoder_t * 
-dv_encoder_new(int rem_ntsc_setup, int clamp_luma, int clamp_chroma) {
+dv_encoder_new(int ignored, int clamp_luma, int clamp_chroma) {
   dv_encoder_t *result;
   
   result = (dv_encoder_t *)calloc(1,sizeof(dv_encoder_t));
   if(!result) return(NULL);
   
+  dv_init( clamp_luma, clamp_chroma);
+
   result->img_y = (short*) calloc(DV_PAL_HEIGHT * DV_WIDTH, sizeof(short));
   if(!result->img_y) goto no_y;
   result->img_cr = (short*) calloc(DV_PAL_HEIGHT * DV_WIDTH / 2, sizeof(short));
@@ -1488,11 +1485,10 @@ dv_encoder_new(int rem_ntsc_setup, int clamp_luma, int clamp_chroma) {
   result->img_cb = (short*) calloc(DV_PAL_HEIGHT * DV_WIDTH / 2, sizeof(short));
   if(!result->img_cb) goto no_cb;
 
-  result->rem_ntsc_setup = rem_ntsc_setup;
+  result->rem_ntsc_setup = FALSE;
   result->clamp_luma = clamp_luma;
   result->clamp_chroma = clamp_chroma;
-
-  dv_init( clamp_luma, clamp_chroma);
+  result->force_dct = DV_DCT_AUTO;
 
   result->frame_count = 0;
   return(result);
@@ -1528,6 +1524,24 @@ dv_encoder_free( dv_encoder_t *encoder)
 } /* dv_encoder_free */
 
 
+/** @brief Free the dynamically allocated global memory.
+ *
+ * This function deallocates the memory allocated by dv_init().
+ *
+ */
+void dv_cleanup(void) {
+  /* the encoder allocates mem on the heap and must be free-ed */
+  if (vlc_encode_lookup != NULL) {
+    free(vlc_encode_lookup);
+    vlc_encode_lookup = NULL;
+  }
+  if (vlc_num_bits_lookup != NULL) {
+    free(vlc_num_bits_lookup);
+    vlc_num_bits_lookup = NULL;
+  }
+} /* dv_cleanup */
+
+
 int dv_encode_videosegment( dv_encoder_t *dv_enc,
 				dv_videosegment_t *videoseg, uint8_t *vsbuffer)
 {
@@ -1605,13 +1619,15 @@ int dv_encode_videosegment( dv_encoder_t *dv_enc,
 	return 0;
 }
 
-
-static void yuy2_to_ycb( uint8_t *data, int isPAL, short *img_y, short *img_cr, short *img_cb)
+/* ---------------------------------------------------------------------------
+ */
+static void
+yuy2_to_ycb( uint8_t *data, int isPAL, short *img_y, short *img_cr, short *img_cb)
 {
 	register int total = (DV_WIDTH * (isPAL ? DV_PAL_HEIGHT : DV_NTSC_HEIGHT)) >> 1;
 	register int i;
 	register uint8_t *p = data;
-	
+
 	for (i = 0; i < total; i++) {
 		img_y[i*2]   = (((short) *p++) - 128) << DCT_YUV_PRECISION;
 		img_cb[i]    = (((short) *p++) - 128) << DCT_YUV_PRECISION;
@@ -1622,17 +1638,29 @@ static void yuy2_to_ycb( uint8_t *data, int isPAL, short *img_y, short *img_cr,
 
 
 #ifdef YUV_420_USE_YV12
-static void yv12_to_ycb( uint8_t **in, int isPAL, short *img_y, short *img_cr, short *img_cb)
+/* ---------------------------------------------------------------------------
+ */
+static void
+yv12_to_ycb( uint8_t **in, int isPAL, short *img_y, short *img_cr, short *img_cb)
 {
-	register int total = (DV_WIDTH * (isPAL ? DV_PAL_HEIGHT : DV_NTSC_HEIGHT)) >> 1;
-	register int i;
+	register int total = DV_WIDTH * (isPAL ? DV_PAL_HEIGHT : DV_NTSC_HEIGHT);
+	register int i, j, k;
 	
-	for (i = 0; i < total; i++) {
-		img_y[i*2]   = (((short) in[0][i*2]) - 128) << DCT_YUV_PRECISION;
-		img_cb[i]    = (((short) in[1][i]) - 128) << DCT_YUV_PRECISION;
-		img_y[i*2+1] = (((short) in[0][i*2+1]) - 128) << DCT_YUV_PRECISION;
-		img_cr[i]    = (((short) in[2][i]) - 128) << DCT_YUV_PRECISION;
-	}
+	
+	for (i = 0; i < total; i++) img_y[i]   = (((short) in[0][i]) - 128) << DCT_YUV_PRECISION;
+	
+	for (i = 0; i < (isPAL ? DV_PAL_HEIGHT : DV_NTSC_HEIGHT)/2; ++i) {
+	  for (j = 0; j < DV_WIDTH/2 ; j++) {
+	    
+	    k = i * DV_WIDTH/2 + j;
+	    
+	    img_cb[2*i*DV_WIDTH/2+j]    = (((short) in[1][k]) - 128) << DCT_YUV_PRECISION;
+	    img_cr[2*i*DV_WIDTH/2+j]    = (((short) in[2][k]) - 128) << DCT_YUV_PRECISION;
+	    img_cb[(2*i+1)*DV_WIDTH/2+j]    = (((short) in[1][k]) - 128) << DCT_YUV_PRECISION;
+	    img_cr[(2*i+1)*DV_WIDTH/2+j]    = (((short) in[2][k]) - 128) << DCT_YUV_PRECISION;
+	  }//col-j
+	}//row-i
+
 }
 #endif
 
@@ -1666,6 +1694,7 @@ static void yv12_to_ycb( uint8_t **in, int isPAL, short *img_y, short *img_cr, s
  * @param out A pointer to the output buffer, which should alreayd be
  *          allocated width*height*2 bytes. The function will clear the buffer
  *          before filling it.
+ * @return -1 for failure, 0 for success
  */
 int dv_encode_full_frame(dv_encoder_t *dv_enc, uint8_t **in,
 			dv_color_space_t color_space, uint8_t *out)
@@ -1792,6 +1821,7 @@ void swab(const void*, void*, ssize_t);
  * @param frequency The sampling rate of the input must be one of 32000,
  *          44100, or 48000.
  * @param frame_buf A pointer to a DV frame
+ * @return -1 for failure, 0 for success
  * 
  * @todo handle 4 channels
  */
